From c3467290925b2a833a68d9db12c198fdca752e32 Mon Sep 17 00:00:00 2001
From: =?UTF-8?q?J=C3=B6rg=20Krause?= <joerg.krause@embedded.rocks>
Date: Sun, 18 Oct 2015 15:54:53 +0200
Subject: [PATCH] Kbuild: fix build issue with some toolchains
MIME-Version: 1.0
Content-Type: text/plain; charset=UTF-8
Content-Transfer-Encoding: 8bit

Some toolchains used in Buildroot (x86-64 and mips64) [1] have a problem to
compile a set of object files into one object file as done for the
`builtin-target`. This targets collects all object files from each subdirectory
declared with `objs-dir`: core and handlers.

For this target the default emulation of ld is not the correct one resulting in
a relocation error:

  /usr/bin/i686-pc-linux-gnu-ld: Relocatable linking with relocations from
  format elf64-x86-64 (core/swupdate.o) to format elf32-i386 (core/built-in.o)
  is not supported.

As there is no need to have `built-in.o` files for core and handlers we can
easily allow swupdate to be built with these toolchains by compiling the source
files in core and handlers with the `lib-target` target as it is done for the
other subdirectories.

The `lib-target` compiles a set of object files into an archive file. With this
target we avoid using the ld linker and use the ar archiver instead.

[1] http://autobuild.buildroot.net/results/d9e/d9e8298898da6355f6b46a032f951b4ca196d103/build-end.log

Signed-off-by: Jörg Krause <joerg.krause@embedded.rocks>
---
 Makefile          |  3 +--
 core/Makefile     |  2 +-
 handlers/Makefile | 14 +++++++-------
 3 files changed, 9 insertions(+), 10 deletions(-)

diff --git a/Makefile b/Makefile
index 3d6a7b7..2dd8c03 100644
--- a/Makefile
+++ b/Makefile
@@ -339,8 +339,7 @@ include $(srctree)/Makefile.flags
 # Defaults to vmlinux, but the arch makefile usually adds further targets
 all: swupdate
 
-objs-y		:= core handlers
-libs-y		:= archival corelib ipc mongoose parser
+libs-y		:= archival core corelib handlers ipc mongoose parser
 
 swupdate-dirs	:= $(objs-y) $(libs-y)
 swupdate-objs	:= $(patsubst %,%/built-in.o, $(objs-y))
diff --git a/core/Makefile b/core/Makefile
index c48e3ee..6302da7 100644
--- a/core/Makefile
+++ b/core/Makefile
@@ -8,7 +8,7 @@
 #
 # Licensed under GPLv2, see file LICENSE in this source tree.
 
-obj-y += swupdate.o \
+lib-y += swupdate.o \
 	 cpio_utils.o \
 	 gpio.o \
 	 notifier.o \
diff --git a/handlers/Makefile b/handlers/Makefile
index 2ea369c..8bd7ea1 100644
--- a/handlers/Makefile
+++ b/handlers/Makefile
@@ -3,10 +3,10 @@
 # not drop the handler if it is not called.
 # Handler can be called dinamically based
 # on the received image type
-obj-$(CONFIG_CFI)	+= flash_handler.o
-obj-$(CONFIG_RAW)	+= raw_handler.o
-obj-$(CONFIG_UBIVOL)	+= ubivol_handler.o
-obj-$(CONFIG_LUASCRIPTHANDLER) += lua_scripthandler.o
-obj-$(CONFIG_SHELLSCRIPTHANDLER) += shell_scripthandler.o
-obj-$(CONFIG_RAW) += raw_handler.o
-obj-$(CONFIG_UBOOT) += uboot_handler.o fw_env.o
+lib-$(CONFIG_CFI)	+= flash_handler.o
+lib-$(CONFIG_RAW)	+= raw_handler.o
+lib-$(CONFIG_UBIVOL)	+= ubivol_handler.o
+lib-$(CONFIG_LUASCRIPTHANDLER) += lua_scripthandler.o
+lib-$(CONFIG_SHELLSCRIPTHANDLER) += shell_scripthandler.o
+lib-$(CONFIG_RAW) += raw_handler.o
+lib-$(CONFIG_UBOOT) += uboot_handler.o fw_env.o
